#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'beautiful_url/class/misc.rb'
# =========================================================================== #
require 'beautiful_url/constants/misc.rb'
require 'beautiful_url/toplevel_methods/current_month.rb'
require 'beautiful_url/toplevel_methods/menu.rb'
require 'beautiful_url/toplevel_methods/moodle_menu.rb'
require 'beautiful_url/toplevel_methods/roebe.rb'

module BeautifulUrl

class BeautifulUrl # === BeautifulUrl::BeautifulUrl

  require 'beautiful_url/class/generate_tab_completion_for_bash_shell.rb'

  begin
    require 'opn'
  rescue LoadError; end

  # ========================================================================= #
  # === map_the_input_arguments_to_their_corresponding_values
  #
  # This method will map the given input to its corresponding value,
  # if possible.
  #
  # Take note that this method will also change @is_registered if the
  # entry was found.
  # ========================================================================= #
  def map_the_input_arguments_to_their_corresponding_values(
      i = @commandline_arguments
    )
    i.map! {|entry|
      if entry.is_a? Symbol
        entry = entry.to_s.dup
      end
      entry = entry.dup if entry.frozen?
      # ===================================================================== #
      # Neither ':' or ',' seem to be useful at the end, so we will chop
      # them away next (since as of October 2011). Trailing whitespace
      # will also be removed in the line afterwards.
      # ===================================================================== #
      if entry and (entry.to_s.end_with?(':') or entry.to_s.end_with?(','))
        entry.chop!
      end
      if entry and entry.is_a?(String)
        entry.rstrip!
        # =================================================================== #
        # Nor is a leading ':' useful here either.
        # =================================================================== #
        entry[0,1] = '' if entry.start_with? ':'
      end
      # ===================================================================== #
      # Handle intralink entries when they contain a '#' character.
      # ===================================================================== #
      if entry and entry.is_a?(String) and entry.include?('#')
        splitted = entry.split('#')
        set_intralink(splitted.last)
        entry = splitted.first
      end
      old_value = entry.to_s.dup # Use a reference copy here.
      new_value = ::BeautifulUrl.menu(entry)
      new_value.flatten! if new_value.is_a? Array
      if new_value == old_value
        # =================================================================== #
        # In this case we did not find anything, so we don't do anything.
        # =================================================================== #
      else
        @is_registered = true
        entry = new_value
      end
      if @intralink and !@intralink.empty?
        entry = entry.dup if entry.frozen?
        entry << @intralink # Append in this case.
      end
      entry
    }
    i.flatten!
  end; alias check_input_against_menu map_the_input_arguments_to_their_corresponding_values # === check_input_against_menu
       alias sanitize_input           map_the_input_arguments_to_their_corresponding_values # === sanitize_input

  # ========================================================================= #
  # === is_a_remote_url?
  #
  # This method will return true if the URL is remote, false if it is
  # local, and nil if it was not found at all.
  #
  # The variable @url_was_found will tell us whether the url was
  # found or whether it was not.
  # ========================================================================= #
  def is_a_remote_url?
    if @is_registered
      _ = @commandline_arguments.all? {|entry|
        entry.include? 'http'
      }
    else
      _ = nil
    end
    _ # Our return value.
  end; alias remote?           is_a_remote_url? # === remote?
       alias is_a_remote_file? is_a_remote_url? # === is_a_remote_file?

  # ========================================================================= #
  # === set_shall_we_replace_localhost
  #
  # This method simply sets @replace_localhost_with_data to the
  # passed value, which is true by default.
  #
  # If true, we will replace the "localhost" string with the
  # DATA constant.
  #
  # Parameters can be like this:
  #
  #   :replace => true     # Yes, replace localhost with data. 
  #   :replace => false    # No, do not replace localhost with data.
  #   :replace => :reverse # Replace data with localhost, which is the reverse.
  #
  # ========================================================================= #
  def set_shall_we_replace_localhost(
      i = true
    )
    @replace_localhost_with_data = i
  end; alias set_replace_localhost       set_shall_we_replace_localhost # === set_replace_localhost
       alias shall_we_replace_localhost= set_shall_we_replace_localhost # === shall_we_replace_localhost=

  # ========================================================================= #
  # === search_for_this_url
  #
  # This will return a string.
  # ========================================================================= #
  def search_for_this_url(i)
    set_commandline_arguments(i)
    map_the_input_arguments_to_their_corresponding_values
    handle_additional_actions_after_the_input_arguments_were_already_mapped
    return string?
  end

  # ========================================================================= #
  # === consider_replacing_data_with_localhost
  # ========================================================================= #
  def consider_replacing_data_with_localhost
    if @replace_data_with_localhost
      do_replace_data_with_localhost
    end
  end

  # ========================================================================= #
  # === consider_replacing_localhost_with_data
  # ========================================================================= #
  def consider_replacing_localhost_with_data
    if @replace_localhost_with_data
      do_replace_localhost_with_data_entry
    end
  end; alias replace_localhost consider_replacing_localhost_with_data # ===replace_localhost

  # ========================================================================= #
  # === handle_additional_actions_after_the_input_arguments_were_already_mapped
  # ========================================================================= #
  def handle_additional_actions_after_the_input_arguments_were_already_mapped
    consider_replacing_localhost_with_data
    consider_replacing_data_with_localhost
  end; alias consider_replacing_everything handle_additional_actions_after_the_input_arguments_were_already_mapped # === consider_replacing_everything
       alias consider_replacing_anything   handle_additional_actions_after_the_input_arguments_were_already_mapped # === consider_replacing_anything

  # ========================================================================= #
  # === do_replace_data_entry_with_localhost
  # ========================================================================= #
  def do_replace_data_entry_with_localhost
    @commandline_arguments.map! {|entry|
      entry = entry.dup if entry.frozen?
      if entry.include? @data_dir
        regexp_quoted = Regexp.quote(@data_dir)
        entry.sub!(regexp_quoted, HTTP_LOCALHOST_STRING)
      end
      entry
    }
  end; alias do_replace_data_with_localhost         do_replace_data_entry_with_localhost # === do_replace_data_with_localhost

  # ========================================================================= #
  # === menu                                                       (menu tag)
  # ========================================================================= #
  def menu(
      i = @commandline_arguments
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i
      # ===================================================================== #
      # === rf --show_help
      # ===================================================================== #
      when /^-?-?show(_|-)?help$/i,
           /^-?-?help$/i
        show_help
      # ===================================================================== #
      # === rf --generate-tab
      # or
      # === purl GENERATE
      # ===================================================================== #
      when 'AUTOGENERATED','GENERATE',
           '--generate',
           '--autogenerate',
           /^-?-?generate?(-|_)?completion$/,
           'generate_shell',
           /^GENE/,
           '--tab',
           '--generate-tab' # Could also be 'GENERATE','GENE'
        generate_tab_completion_for_bash_shell # This here is in another file.
        exit
      end
    end
  end

  # ========================================================================= #
  # === is_registered?
  # ========================================================================= #
  def is_registered?
    @is_registered
  end; alias url_was_found? is_registered? # === url_was_found?
       alias url_was_found  is_registered? # === url_was_found
       alias is_included?   is_registered? # === is_included?

  # ========================================================================= #
  # === do_replace_localhost_with_data_entry
  #
  # This method will replace the leading localhost-string with
  # the corresponding data-dir value.
  #
  # For example, the :
  #
  #   http://localhost/DATA/PC/palemoon/palemoon.cgi
  #
  # Would become:
  #
  #   /home/x/DATA/PC/palemoon/palemoon.cgi
  #
  # ========================================================================= #
  def do_replace_localhost_with_data_entry
    @commandline_arguments.map! {|entry|
      entry = entry.dup if entry.frozen?
      if entry.include? HTTP_LOCALHOST_STRING
        regexp_quoted = Regexp.quote(HTTP_LOCALHOST_STRING)
        entry.sub!(regexp_quoted, home_dir_of_user_x?)
      end
      entry
    }
  end; alias get_rid_of_localhost           do_replace_localhost_with_data_entry # === get_rid_of_localhost
       alias do_replace_localhost_with_data do_replace_localhost_with_data_entry # === do_replace_localhost_with_data
       alias do_replace_localhost           do_replace_localhost_with_data_entry # === do_replace_localhost

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = :default)
    case i
    when nil
      i = DEFAULT_URL
    end
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === replace_localhost_with_data?
  # ========================================================================= #
  def replace_localhost_with_data?
    @replace_localhost_with_data
  end; alias replace_localhost_with_data replace_localhost_with_data? # === replace_localhost_with_data

  # ========================================================================= #
  # === set_intralink
  #
  # This method sets the @intralink variable. It is the only allowed
  # way to modify this variable. The @intralink variable can be a
  # pointer to a local or to a remote page.
  #
  # As of August 2014, we will only set @intralink if we provide input.
  # ========================================================================= #
  def set_intralink(
      i = nil, use_this_token = '#'
    )
    case i
    when :default
      i = nil
    end
    if i
      i[0,0] = use_this_token unless i.empty?
    end
    @intralink = i # Must always be set, even to nil.
  end

  # ========================================================================= #
  # === set_data_dir
  #
  # We need this method so that the user can define another data-dir
  # that is to be used.
  # ========================================================================= #
  def set_data_dir(
      i = :default
    )
    case i
    # ======================================================================= #
    # === :default
    # ======================================================================= #
    when :default,
         nil
      i = DATA_DIR
    end
    @data_dir = i
  end

  # ========================================================================= #
  # === return_corresponding_entries
  # ========================================================================= #
  def return_corresponding_entries
    @commandline_arguments
  end; alias input? return_corresponding_entries # === input?
       alias data   return_corresponding_entries # === data
       alias array  return_corresponding_entries # === array
       alias array? return_corresponding_entries # === array?

  # ========================================================================= #
  # === return_corresponding_entry
  # ========================================================================= #
  def return_corresponding_entry
    @commandline_arguments.first
  end; alias url? return_corresponding_entry # === url?

  # ========================================================================= #
  # === results?
  # ========================================================================= #
  def results?
    @commandline_arguments
  end

  # ========================================================================= #
  # === show_result
  # ========================================================================= #
  def show_result
    e return_corresponding_entry
  end

  # ========================================================================= #
  # === home_dir_of_user_x?
  # ========================================================================= #
  def home_dir_of_user_x?
    BASE_DIR
  end

  # ========================================================================= #
  # === do_ignore_help
  # ========================================================================= #
  def do_ignore_help
    @ignore_help = true
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help(
      shall_we_exit = :then_exit
    )
    unless @ignore_help
      opn
      e 'To issue the various instructions, use AUTOGENERATED or GENERATED.'
      e
      e 'Alternative, it works as a downcased variant as well, with -- as in:'
      e
      e '  --autogenerate'
      e '  --generate'
      e
      exit if shall_we_exit == :then_exit
    end
  end

  # ========================================================================= #
  # === url_as_string
  #
  # Return the @input in string variant, always. This can be used as
  # content that can be displayed in other programs.
  #
  # Take note that in the past result? returned an Array, from 02.08.2015
  # up until 04.01.2020. Past that point it will return a String instead.
  # ========================================================================= #
  def url_as_string
    return_corresponding_entry.to_s
  end; alias url?             url_as_string # === url?
       alias location?        url_as_string # === location?
       alias string           url_as_string # === string
       alias string?          url_as_string # === string?
       alias result?          url_as_string # === result?
       alias result           url_as_string # === result
       alias return_as_string url_as_string # === return_as_string
       alias []               url_as_string # === []

  # ========================================================================= #
  # === BeautifulUrl::BeautifulUrl['palemoon']
  # ========================================================================= #
  def self.[](
      i = 'palemoon', &block
    )
    new(i, &block).return_as_string
  end

end

# =========================================================================== #
# === BeautifulUrl.fetch_value
#
# This method simply allows you to return the URL on the commandline. It
# will also set the xorg-buffer, if available, so it is ideally used
# for commandline-applications, e. g. if the xorg-server is not available.
#
# Usage example would be:
#
#   BeautifulUrl.fetch_value 'php' # => "/home/x/DATA/PROGRAMMING_LANGUAGES/php/PHP_TUTORIAL.php"
#
# =========================================================================== #
def self.fetch_value(
    i
  )
  result = ::BeautifulUrl::BeautifulUrl.new(i).result?
  if Object.const_defined? :XorgBuffer
    XorgBuffer[result]
  end
  return result
end

# ========================================================================= #
# === BeautifulUrl.is_included?
#
# Use this method to find out whether an entry is registered or whether
# it is not.
#
# Usage example:
#
#   x = BeautifulUrl.is_included? 'htop'
#
# ========================================================================= #
def self.is_included?(i)
  # ======================================================================= #
  # The next check was added in September 2015. It was disabled again
  # in 2020.
  # ======================================================================= #
  # i = i.to_sym unless i.is_a? Symbol
  _ = ::BeautifulUrl::BeautifulUrl.new(i)
  _.is_included? # Work on the class-level there.
end; self.instance_eval { alias include?        is_included? } # === BeautifulUrl.include?
     self.instance_eval { alias has_this_entry? is_included? } # === BeautifulUrl.has_this_entry?
     self.instance_eval { alias does_include?   is_included? } # === BeautifulUrl.does_include?

# ========================================================================= #
# === BeautifulUrl.replace_localhost
# ========================================================================= #
def self.replace_localhost(i)
  ::BeautifulUrl::BeautifulUrl.new(i) { :replace_localhost }
end

# =========================================================================== #
# === BeautifulUrl.map
#
# Usage examples:
#
#   BeautifulUrl.map('palemoon')   # => "http://localhost/DATA/PC/palemoon/palemoon.cgi"
#   BeautifulUrl.parse('palemoon') # => "http://localhost/DATA/PC/palemoon/palemoon.cgi"
#   BeautifulUrl.parse(:remote_bioroebe) # => "https://rubygems.org/gems/bioroebe"
#   BeautifulUrl.parse(:a_fat_cat) # => "https://i.imgur.com/h8CTcxq.png"
#
# =========================================================================== #
def self.map(
    i, &block
  )
  ::BeautifulUrl::BeautifulUrl.new(i, &block).return_as_string
end; self.instance_eval { alias []               map } # === BeautifulUrl[]
     self.instance_eval { alias convert          map } # === BeautifulUrl.convert
     self.instance_eval { alias new              map } # === BeautifulUrl.new
     self.instance_eval { alias return_url_from  map } # === BeautifulUrl.return_url_from
     self.instance_eval { alias return_as_string map } # === BeautifulUrl.return_as_string
     self.instance_eval { alias parse            map } # === BeautifulUrl.parse
     self.instance_eval { alias symbol_to_url    map } # === BeautifulUrl.symbol_to_url
     self.instance_eval { alias this_remote_URL  map } # === BeautifulUrl.this_remote_URL

end